@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /* Implementation of SMPTE 384M - Mapping of Uncompressed Pictures into the MXF
@@ -46,56 +46,54 @@ GST_DEBUG_CATEGORY_EXTERN (mxf_debug);
 
 static const struct
 {
-  const gchar *caps;
+  const gchar *format;
   guint32 n_pixel_layout;
   guint8 pixel_layout[10];
-  guint32 fourcc;
+  const gchar *caps_string;
 } _rgba_mapping_table[] = {
   {
-    GST_VIDEO_CAPS_RGB, 3, {
-  'R', 8, 'G', 8, 'B', 8}, GST_MAKE_FOURCC ('R', 'G', 'B', ' ')}, {
-    GST_VIDEO_CAPS_BGR, 3, {
-  'B', 8, 'G', 8, 'R', 8}, GST_MAKE_FOURCC ('B', 'G', 'R', ' ')}, {
-    GST_VIDEO_CAPS_YUV ("v308"), 3, {
-  'Y', 8, 'U', 8, 'V', 8}, GST_MAKE_FOURCC ('v', '3', '0', '8')}, {
-    GST_VIDEO_CAPS_xRGB, 4, {
-  'F', 8, 'R', 8, 'G', 8, 'B', 8}, GST_MAKE_FOURCC ('x', 'R', 'G', 'B')}, {
-    GST_VIDEO_CAPS_RGBx, 4, {
-  'R', 8, 'G', 8, 'B', 8, 'F', 8}, GST_MAKE_FOURCC ('R', 'G', 'B', 'x')}, {
-    GST_VIDEO_CAPS_xBGR, 4, {
-  'F', 8, 'B', 8, 'G', 8, 'R', 8}, GST_MAKE_FOURCC ('x', 'B', 'G', 'R')}, {
-    GST_VIDEO_CAPS_BGRx, 4, {
-  'B', 8, 'G', 8, 'R', 8, 'F', 8}, GST_MAKE_FOURCC ('B', 'G', 'R', 'x')}, {
-    GST_VIDEO_CAPS_RGBA, 4, {
-  'R', 8, 'G', 8, 'B', 8, 'A', 8}, GST_MAKE_FOURCC ('R', 'G', 'B', 'A')}, {
-    GST_VIDEO_CAPS_ARGB, 4, {
-  'A', 8, 'R', 8, 'G', 8, 'B', 8}, GST_MAKE_FOURCC ('A', 'R', 'G', 'B')}, {
-    GST_VIDEO_CAPS_BGRA, 4, {
-  'B', 8, 'G', 8, 'R', 8, 'A', 8}, GST_MAKE_FOURCC ('B', 'G', 'R', 'A')}, {
-    GST_VIDEO_CAPS_ABGR, 4, {
-  'A', 8, 'B', 8, 'G', 8, 'R', 8}, GST_MAKE_FOURCC ('A', 'B', 'G', 'R')}, {
-    GST_VIDEO_CAPS_YUV ("AYUV"), 4, {
-  'A', 8, 'Y', 8, 'U', 8, 'V', 8}, GST_MAKE_FOURCC ('A', 'Y', 'U', 'V')}
+    "RGB", 3, {
+  'R', 8, 'G', 8, 'B', 8}, GST_VIDEO_CAPS_MAKE ("RGB")}, {
+    "BGR", 3, {
+  'B', 8, 'G', 8, 'R', 8}, GST_VIDEO_CAPS_MAKE ("BGR")}, {
+    "v308", 3, {
+  'Y', 8, 'U', 8, 'V', 8}, GST_VIDEO_CAPS_MAKE ("v308")}, {
+    "xRGB", 4, {
+  'F', 8, 'R', 8, 'G', 8, 'B', 8}, GST_VIDEO_CAPS_MAKE ("xRGB")}, {
+    "RGBx", 4, {
+  'R', 8, 'G', 8, 'B', 8, 'F', 8}, GST_VIDEO_CAPS_MAKE ("RGBx")}, {
+    "xBGR", 4, {
+  'F', 8, 'B', 8, 'G', 8, 'R', 8}, GST_VIDEO_CAPS_MAKE ("xBGR")}, {
+    "BGRx", 4, {
+  'B', 8, 'G', 8, 'R', 8, 'F', 8}, GST_VIDEO_CAPS_MAKE ("BGRx")}, {
+    "RGBA", 4, {
+  'R', 8, 'G', 8, 'B', 8, 'A', 8}, GST_VIDEO_CAPS_MAKE ("RGBA")}, {
+    "ARGB", 4, {
+  'A', 8, 'R', 8, 'G', 8, 'B', 8}, GST_VIDEO_CAPS_MAKE ("RGBA")}, {
+    "BGRA", 4, {
+  'B', 8, 'G', 8, 'R', 8, 'A', 8}, GST_VIDEO_CAPS_MAKE ("BGRA")}, {
+    "ABGR", 4, {
+  'A', 8, 'B', 8, 'G', 8, 'R', 8}, GST_VIDEO_CAPS_MAKE ("ABGR")}, {
+    "AYUV", 4, {
+  'A', 8, 'Y', 8, 'U', 8, 'V', 8}, GST_VIDEO_CAPS_MAKE ("AYUV")}
 };
 
 static const struct
 {
-  const gchar *caps;
+  const gchar *format;
   guint bpp;
   guint horizontal_subsampling;
   guint vertical_subsampling;
   gboolean reversed_byte_order;
-  guint32 fourcc;
+  const gchar *caps_string;
 } _cdci_mapping_table[] = {
   {
-  GST_VIDEO_CAPS_YUV ("YUY2"), 2, 1, 0, TRUE, GST_MAKE_FOURCC ('Y', 'U', 'Y',
-        '2')}, {
-GST_VIDEO_CAPS_YUV ("UYVY"), 2, 1, 0, FALSE, GST_MAKE_FOURCC ('U', 'Y', 'V',
-        'Y')},};
+  "YUY2", 2, 1, 0, TRUE, GST_VIDEO_CAPS_MAKE ("YUY2")}, {
+"UYVY", 2, 1, 0, FALSE, GST_VIDEO_CAPS_MAKE ("UYVY")},};
 
 typedef struct
 {
-  guint32 fourcc;               /* fourcc or RGB format specifier */
+  const gchar *format;          /* video format string */
   gint width, height;
   guint bpp;
   guint32 image_start_offset;
@@ -136,51 +134,69 @@ mxf_up_handle_essence_element (const MXFUL * key, GstBuffer * buffer,
     gpointer mapping_data, GstBuffer ** outbuf)
 {
   MXFUPMappingData *data = mapping_data;
+  gsize expected_in_stride = 0, out_stride = 0;
+  gsize expected_in_size = 0, out_size = 0;
 
   /* SMPTE 384M 7.1 */
   if (key->u[12] != 0x15 || (key->u[14] != 0x01 && key->u[14] != 0x02
           && key->u[14] != 0x03 && key->u[14] != 0x04)) {
     GST_ERROR ("Invalid uncompressed picture essence element");
+    gst_buffer_unref (buffer);
     return GST_FLOW_ERROR;
   }
 
-  if (!data || (data->image_start_offset == 0 && data->image_end_offset == 0)) {
+  if (!data) {
+    GST_ERROR ("Invalid mapping data");
+    gst_buffer_unref (buffer);
+    return GST_FLOW_ERROR;
+  }
+
+  if (data->image_start_offset == 0 && data->image_end_offset == 0) {
   } else {
     if (data->image_start_offset + data->image_end_offset
-        > GST_BUFFER_SIZE (buffer)) {
+        > gst_buffer_get_size (buffer)) {
       gst_buffer_unref (buffer);
       GST_ERROR ("Invalid buffer size");
       return GST_FLOW_ERROR;
     } else {
-      GST_BUFFER_DATA (buffer) += data->image_start_offset;
-      GST_BUFFER_SIZE (buffer) -= data->image_start_offset;
-      GST_BUFFER_SIZE (buffer) -= data->image_end_offset;
+      gst_buffer_resize (buffer, data->image_start_offset,
+          data->image_end_offset - data->image_start_offset);
     }
   }
 
-  if (GST_BUFFER_SIZE (buffer) != data->bpp * data->width * data->height) {
+  // Checked for overflows when parsing the descriptor
+  expected_in_stride = data->bpp * data->width;
+  out_stride = GST_ROUND_UP_4 (expected_in_stride);
+  expected_in_size = expected_in_stride * data->height;
+  out_size = out_stride * data->height;
+
+  if (gst_buffer_get_size (buffer) != expected_in_size) {
     GST_ERROR ("Invalid buffer size");
+    gst_buffer_unref (buffer);
     return GST_FLOW_ERROR;
   }
 
-  if (data->bpp != 4
-      || GST_ROUND_UP_4 (data->width * data->bpp) != data->width * data->bpp) {
+  if (data->bpp != 4 || out_stride != expected_in_stride) {
     guint y;
     GstBuffer *ret;
+    GstMapInfo inmap, outmap;
     guint8 *indata, *outdata;
 
-    ret =
-        gst_buffer_new_and_alloc (GST_ROUND_UP_4 (data->width * data->bpp) *
-        data->height);
-    indata = GST_BUFFER_DATA (buffer);
-    outdata = GST_BUFFER_DATA (ret);
+    ret = gst_buffer_new_and_alloc (out_size);
+    gst_buffer_map (buffer, &inmap, GST_MAP_READ);
+    gst_buffer_map (ret, &outmap, GST_MAP_WRITE);
+    indata = inmap.data;
+    outdata = outmap.data;
 
     for (y = 0; y < data->height; y++) {
       memcpy (outdata, indata, data->width * data->bpp);
-      outdata += GST_ROUND_UP_4 (data->width * data->bpp);
-      indata += data->width * data->bpp;
+      outdata += out_stride;
+      indata += expected_in_stride;
     }
 
+    gst_buffer_unmap (buffer, &inmap);
+    gst_buffer_unmap (ret, &outmap);
+
     gst_buffer_unref (buffer);
     *outbuf = ret;
   } else {
@@ -190,14 +206,51 @@ mxf_up_handle_essence_element (const MXFUL * key, GstBuffer * buffer,
   return GST_FLOW_OK;
 }
 
+static MXFEssenceWrapping
+mxf_up_get_track_wrapping (const MXFMetadataTimelineTrack * track)
+{
+  guint i;
+
+  g_return_val_if_fail (track != NULL, MXF_ESSENCE_WRAPPING_CUSTOM_WRAPPING);
+
+  if (track->parent.descriptor == NULL) {
+    GST_ERROR ("No descriptor found for this track");
+    return MXF_ESSENCE_WRAPPING_CUSTOM_WRAPPING;
+  }
+
+  for (i = 0; i < track->parent.n_descriptor; i++) {
+    if (!track->parent.descriptor[i])
+      continue;
+
+    if (!MXF_IS_METADATA_GENERIC_PICTURE_ESSENCE_DESCRIPTOR (track->
+            parent.descriptor[i]))
+      continue;
+
+    switch (track->parent.descriptor[i]->essence_container.u[15]) {
+      case 0x01:
+        return MXF_ESSENCE_WRAPPING_FRAME_WRAPPING;
+        break;
+      case 0x02:
+        return MXF_ESSENCE_WRAPPING_CLIP_WRAPPING;
+        break;
+      default:
+        return MXF_ESSENCE_WRAPPING_CUSTOM_WRAPPING;
+        break;
+    }
+  }
+
+  return MXF_ESSENCE_WRAPPING_CUSTOM_WRAPPING;
+}
+
 static GstCaps *
 mxf_up_rgba_create_caps (MXFMetadataTimelineTrack * track,
     MXFMetadataRGBAPictureEssenceDescriptor * d, GstTagList ** tags,
-    MXFEssenceElementHandleFunc * handler, gpointer * mapping_data)
+    gboolean * intra_only, MXFEssenceElementHandleFunc * handler,
+    gpointer * mapping_data)
 {
   GstCaps *caps = NULL;
   guint i;
-  guint32 fourcc;
+  const gchar *format = NULL;
   guint bpp;
 
   if (!d->pixel_layout) {
@@ -211,8 +264,8 @@ mxf_up_rgba_create_caps (MXFMetadataTimelineTrack * track,
 
     if (memcmp (d->pixel_layout, &_rgba_mapping_table[i].pixel_layout,
             _rgba_mapping_table[i].n_pixel_layout * 2) == 0) {
-      caps = gst_caps_from_string (_rgba_mapping_table[i].caps);
-      fourcc = _rgba_mapping_table[i].fourcc;
+      caps = gst_caps_from_string (_rgba_mapping_table[i].caps_string);
+      format = _rgba_mapping_table[i].format;
       bpp = _rgba_mapping_table[i].n_pixel_layout;
       break;
     }
@@ -225,7 +278,7 @@ mxf_up_rgba_create_caps (MXFMetadataTimelineTrack * track,
 
     data->width = d->parent.stored_width;
     data->height = d->parent.stored_height;
-    data->fourcc = fourcc;
+    data->format = format;
     data->bpp = bpp;
     data->image_start_offset =
         ((MXFMetadataGenericPictureEssenceDescriptor *) d)->image_start_offset;
@@ -233,6 +286,7 @@ mxf_up_rgba_create_caps (MXFMetadataTimelineTrack * track,
         ((MXFMetadataGenericPictureEssenceDescriptor *) d)->image_end_offset;
 
     *mapping_data = data;
+    *intra_only = TRUE;
   } else {
     GST_WARNING ("Unsupported pixel layout");
   }
@@ -243,11 +297,12 @@ mxf_up_rgba_create_caps (MXFMetadataTimelineTrack * track,
 static GstCaps *
 mxf_up_cdci_create_caps (MXFMetadataTimelineTrack * track,
     MXFMetadataCDCIPictureEssenceDescriptor * d, GstTagList ** tags,
-    MXFEssenceElementHandleFunc * handler, gpointer * mapping_data)
+    gboolean * intra_only, MXFEssenceElementHandleFunc * handler,
+    gpointer * mapping_data)
 {
   GstCaps *caps = NULL;
   guint i;
-  guint32 fourcc;
+  const gchar *format;
   guint bpp;
 
   for (i = 0; i < G_N_ELEMENTS (_cdci_mapping_table); i++) {
@@ -257,8 +312,8 @@ mxf_up_cdci_create_caps (MXFMetadataTimelineTrack * track,
         d->vertical_subsampling
         && _cdci_mapping_table[i].reversed_byte_order ==
         d->reversed_byte_order) {
-      caps = gst_caps_from_string (_cdci_mapping_table[i].caps);
-      fourcc = _cdci_mapping_table[i].fourcc;
+      caps = gst_caps_from_string (_cdci_mapping_table[i].caps_string);
+      format = _cdci_mapping_table[i].format;
       bpp = _cdci_mapping_table[i].bpp;
       break;
     }
@@ -271,7 +326,7 @@ mxf_up_cdci_create_caps (MXFMetadataTimelineTrack * track,
 
     data->width = d->parent.stored_width;
     data->height = d->parent.stored_height;
-    data->fourcc = fourcc;
+    data->format = format;
     data->bpp = bpp;
     data->image_start_offset =
         ((MXFMetadataGenericPictureEssenceDescriptor *) d)->image_start_offset;
@@ -279,6 +334,7 @@ mxf_up_cdci_create_caps (MXFMetadataTimelineTrack * track,
         ((MXFMetadataGenericPictureEssenceDescriptor *) d)->image_end_offset;
 
     *mapping_data = data;
+    *intra_only = TRUE;
   } else {
     GST_WARNING ("Unsupported CDCI format");
   }
@@ -288,7 +344,8 @@ mxf_up_cdci_create_caps (MXFMetadataTimelineTrack * track,
 
 static GstCaps *
 mxf_up_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
-    MXFEssenceElementHandleFunc * handler, gpointer * mapping_data)
+    gboolean * intra_only, MXFEssenceElementHandleFunc * handler,
+    gpointer * mapping_data)
 {
   MXFMetadataGenericPictureEssenceDescriptor *p = NULL;
   MXFMetadataCDCIPictureEssenceDescriptor *c = NULL;
@@ -307,19 +364,19 @@ mxf_up_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
     if (!track->parent.descriptor[i])
       continue;
 
-    if (MXF_IS_METADATA_RGBA_PICTURE_ESSENCE_DESCRIPTOR (track->
-            parent.descriptor[i])) {
-      p = (MXFMetadataGenericPictureEssenceDescriptor *) track->
-          parent.descriptor[i];
-      r = (MXFMetadataRGBAPictureEssenceDescriptor *) track->
-          parent.descriptor[i];
-      break;
-    } else if (MXF_IS_METADATA_CDCI_PICTURE_ESSENCE_DESCRIPTOR (track->parent.
+    if (MXF_IS_METADATA_RGBA_PICTURE_ESSENCE_DESCRIPTOR (track->parent.
             descriptor[i])) {
-      p = (MXFMetadataGenericPictureEssenceDescriptor *) track->
-          parent.descriptor[i];
-      c = (MXFMetadataCDCIPictureEssenceDescriptor *) track->
-          parent.descriptor[i];
+      p = (MXFMetadataGenericPictureEssenceDescriptor *) track->parent.
+          descriptor[i];
+      r = (MXFMetadataRGBAPictureEssenceDescriptor *) track->parent.
+          descriptor[i];
+      break;
+    } else if (MXF_IS_METADATA_CDCI_PICTURE_ESSENCE_DESCRIPTOR (track->
+            parent.descriptor[i])) {
+      p = (MXFMetadataGenericPictureEssenceDescriptor *) track->parent.
+          descriptor[i];
+      c = (MXFMetadataCDCIPictureEssenceDescriptor *) track->parent.
+          descriptor[i];
     }
   }
 
@@ -331,23 +388,58 @@ mxf_up_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
   *handler = mxf_up_handle_essence_element;
 
   if (r) {
-    caps = mxf_up_rgba_create_caps (track, r, tags, handler, mapping_data);
+    caps =
+        mxf_up_rgba_create_caps (track, r, tags, intra_only, handler,
+        mapping_data);
   } else if (c) {
-    caps = mxf_up_cdci_create_caps (track, c, tags, handler, mapping_data);
+    caps =
+        mxf_up_cdci_create_caps (track, c, tags, intra_only, handler,
+        mapping_data);
   } else {
     return NULL;
   }
 
+  if (caps) {
+    MXFUPMappingData *data = *mapping_data;
+    gsize expected_in_stride = 0, out_stride = 0;
+    gsize expected_in_size = 0, out_size = 0;
+
+    // Do some checking of the parameters to see if they're valid and
+    // we can actually work with them.
+    if (data->image_start_offset > data->image_end_offset) {
+      GST_WARNING ("Invalid image start/end offset");
+      g_free (data);
+      *mapping_data = NULL;
+      gst_clear_caps (&caps);
+
+      return NULL;
+    }
+
+    if (!g_size_checked_mul (&expected_in_stride, data->bpp, data->width) ||
+        (out_stride = GST_ROUND_UP_4 (expected_in_stride)) < expected_in_stride
+        || !g_size_checked_mul (&expected_in_size, expected_in_stride,
+            data->height)
+        || !g_size_checked_mul (&out_size, out_stride, data->height)) {
+      GST_ERROR ("Invalid resolution or bit depth");
+      g_free (data);
+      *mapping_data = NULL;
+      gst_clear_caps (&caps);
+
+      return NULL;
+    }
+  }
+
   return caps;
 }
 
 static const MXFEssenceElementHandler mxf_up_essence_element_handler = {
   mxf_is_up_essence_track,
+  mxf_up_get_track_wrapping,
   mxf_up_create_caps
 };
 
 static GstFlowReturn
-mxf_up_write_func (GstBuffer * buffer, GstCaps * caps, gpointer mapping_data,
+mxf_up_write_func (GstBuffer * buffer, gpointer mapping_data,
     GstAdapter * adapter, GstBuffer ** outbuf, gboolean flush)
 {
   MXFUPMappingData *data = mapping_data;
@@ -355,7 +447,7 @@ mxf_up_write_func (GstBuffer * buffer, GstCaps * caps, gpointer mapping_data,
   if (!buffer)
     return GST_FLOW_OK;
 
-  if (GST_BUFFER_SIZE (buffer) !=
+  if (gst_buffer_get_size (buffer) !=
       GST_ROUND_UP_4 (data->bpp * data->width) * data->height) {
     GST_ERROR ("Invalid buffer size");
     return GST_FLOW_ERROR;
@@ -365,11 +457,14 @@ mxf_up_write_func (GstBuffer * buffer, GstCaps * caps, gpointer mapping_data,
       || GST_ROUND_UP_4 (data->width * data->bpp) != data->width * data->bpp) {
     guint y;
     GstBuffer *ret;
+    GstMapInfo inmap, outmap;
     guint8 *indata, *outdata;
 
     ret = gst_buffer_new_and_alloc (data->width * data->bpp * data->height);
-    indata = GST_BUFFER_DATA (buffer);
-    outdata = GST_BUFFER_DATA (ret);
+    gst_buffer_map (buffer, &inmap, GST_MAP_READ);
+    gst_buffer_map (ret, &outmap, GST_MAP_WRITE);
+    indata = inmap.data;
+    outdata = outmap.data;
 
     for (y = 0; y < data->height; y++) {
       memcpy (outdata, indata, data->width * data->bpp);
@@ -377,6 +472,8 @@ mxf_up_write_func (GstBuffer * buffer, GstCaps * caps, gpointer mapping_data,
       outdata += data->width * data->bpp;
     }
 
+    gst_buffer_unmap (buffer, &inmap);
+    gst_buffer_unmap (ret, &outmap);
     gst_buffer_unref (buffer);
 
     *outbuf = ret;
@@ -407,7 +504,7 @@ mxf_up_get_rgba_descriptor (GstPadTemplate * tmpl, GstCaps * caps,
       g_object_new (MXF_TYPE_METADATA_RGBA_PICTURE_ESSENCE_DESCRIPTOR, NULL);
 
   for (i = 0; i < G_N_ELEMENTS (_rgba_mapping_table); i++) {
-    tmp = gst_caps_from_string (_rgba_mapping_table[i].caps);
+    tmp = gst_caps_from_string (_rgba_mapping_table[i].caps_string);
     intersection = gst_caps_intersect (caps, tmp);
     gst_caps_unref (tmp);
 
@@ -415,7 +512,7 @@ mxf_up_get_rgba_descriptor (GstPadTemplate * tmpl, GstCaps * caps,
       gst_caps_unref (intersection);
       ret->n_pixel_layout = _rgba_mapping_table[i].n_pixel_layout;
       ret->pixel_layout = g_new0 (guint8, ret->n_pixel_layout * 2);
-      md->fourcc = _rgba_mapping_table[i].fourcc;
+      md->format = _rgba_mapping_table[i].format;
       md->bpp = _rgba_mapping_table[i].n_pixel_layout;
       memcpy (ret->pixel_layout, _rgba_mapping_table[i].pixel_layout,
           ret->n_pixel_layout * 2);
@@ -424,7 +521,7 @@ mxf_up_get_rgba_descriptor (GstPadTemplate * tmpl, GstCaps * caps,
     gst_caps_unref (intersection);
   }
 
-  if (md->fourcc == 0) {
+  if (md->format == NULL) {
     GST_ERROR ("Invalid caps %" GST_PTR_FORMAT, caps);
     g_object_unref (ret);
     return NULL;
@@ -462,7 +559,7 @@ mxf_up_get_cdci_descriptor (GstPadTemplate * tmpl, GstCaps * caps,
       g_object_new (MXF_TYPE_METADATA_CDCI_PICTURE_ESSENCE_DESCRIPTOR, NULL);
 
   for (i = 0; i < G_N_ELEMENTS (_cdci_mapping_table); i++) {
-    tmp = gst_caps_from_string (_cdci_mapping_table[i].caps);
+    tmp = gst_caps_from_string (_cdci_mapping_table[i].caps_string);
     intersection = gst_caps_intersect (caps, tmp);
     gst_caps_unref (tmp);
 
@@ -472,14 +569,14 @@ mxf_up_get_cdci_descriptor (GstPadTemplate * tmpl, GstCaps * caps,
           _cdci_mapping_table[i].horizontal_subsampling;
       ret->vertical_subsampling = _cdci_mapping_table[i].vertical_subsampling;
       ret->reversed_byte_order = _cdci_mapping_table[i].reversed_byte_order;
-      md->fourcc = _cdci_mapping_table[i].fourcc;
+      md->format = _cdci_mapping_table[i].format;
       md->bpp = _cdci_mapping_table[i].bpp;
       break;
     }
     gst_caps_unref (intersection);
   }
 
-  if (md->fourcc == 0) {
+  if (md->format == NULL) {
     GST_ERROR ("Invalid caps %" GST_PTR_FORMAT, caps);
     g_object_unref (ret);
     return NULL;
@@ -508,19 +605,17 @@ mxf_up_get_descriptor (GstPadTemplate * tmpl, GstCaps * caps,
   GstStructure *s;
 
   s = gst_caps_get_structure (caps, 0);
-  if (strcmp (gst_structure_get_name (s), "video/x-raw-rgb") == 0) {
-    return mxf_up_get_rgba_descriptor (tmpl, caps, handler, mapping_data);
-  } else if (strcmp (gst_structure_get_name (s), "video/x-raw-yuv") == 0) {
-    guint32 fourcc;
+  if (strcmp (gst_structure_get_name (s), "video/x-raw") == 0) {
+    const gchar *format;
 
-    if (!gst_structure_get_fourcc (s, "format", &fourcc))
+    format = gst_structure_get_string (s, "format");
+    if (format == NULL)
       return NULL;
 
-    if (fourcc == GST_MAKE_FOURCC ('A', 'Y', 'U', 'V') ||
-        fourcc == GST_MAKE_FOURCC ('v', '3', '0', '8'))
+    if (g_str_equal (format, "YUY2") || g_str_equal (format, "UYVY"))
+      return mxf_up_get_cdci_descriptor (tmpl, caps, handler, mapping_data);
+    else
       return mxf_up_get_rgba_descriptor (tmpl, caps, handler, mapping_data);
-
-    return mxf_up_get_cdci_descriptor (tmpl, caps, handler, mapping_data);
   }
 
   g_assert_not_reached ();
@@ -563,22 +658,21 @@ void
 mxf_up_init (void)
 {
   mxf_essence_element_handler_register (&mxf_up_essence_element_handler);
-
   mxf_up_essence_element_writer.pad_template =
       gst_pad_template_new ("up_video_sink_%u", GST_PAD_SINK, GST_PAD_REQUEST,
-      gst_caps_from_string (GST_VIDEO_CAPS_RGB "; "
-          GST_VIDEO_CAPS_BGR "; "
-          GST_VIDEO_CAPS_RGBx "; "
-          GST_VIDEO_CAPS_xRGB "; "
-          GST_VIDEO_CAPS_BGRx "; "
-          GST_VIDEO_CAPS_xBGR "; "
-          GST_VIDEO_CAPS_ARGB "; "
-          GST_VIDEO_CAPS_RGBA "; "
-          GST_VIDEO_CAPS_ABGR "; "
-          GST_VIDEO_CAPS_BGRA "; "
-          GST_VIDEO_CAPS_YUV ("AYUV") "; "
-          GST_VIDEO_CAPS_YUV ("v308") "; "
-          GST_VIDEO_CAPS_YUV ("UYVY") "; " GST_VIDEO_CAPS_YUV ("YUY2")));
+      gst_caps_from_string (GST_VIDEO_CAPS_MAKE ("RGB") "; "
+          GST_VIDEO_CAPS_MAKE ("BGR") "; "
+          GST_VIDEO_CAPS_MAKE ("RGBx") "; "
+          GST_VIDEO_CAPS_MAKE ("xRGB") "; "
+          GST_VIDEO_CAPS_MAKE ("BGRx") "; "
+          GST_VIDEO_CAPS_MAKE ("xBGR") "; "
+          GST_VIDEO_CAPS_MAKE ("ARGB") "; "
+          GST_VIDEO_CAPS_MAKE ("RGBA") "; "
+          GST_VIDEO_CAPS_MAKE ("ABGR") "; "
+          GST_VIDEO_CAPS_MAKE ("BGRA") "; "
+          GST_VIDEO_CAPS_MAKE ("AYUV") "; "
+          GST_VIDEO_CAPS_MAKE ("v308") "; "
+          GST_VIDEO_CAPS_MAKE ("UYVY") "; " GST_VIDEO_CAPS_MAKE ("YUY2")));
 
   memcpy (&mxf_up_essence_element_writer.data_definition,
       mxf_metadata_track_identifier_get (MXF_METADATA_TRACK_PICTURE_ESSENCE),
